<?php

namespace Drupal\remote_file_importer_ftp\Plugin\RemoteFileImporter\Datasource;

use Drupal\remote_file_importer\DataSourcePluginBase;

/**
 * Provids FTP import functionality.
 *
 * @RemoteFileImporterDataSourcePlugin(
 *   id = "rif_ftp_datasource",
 *   label = @Translation("FTP"),
 * )
 */
class FtpDataSource extends DataSourcePluginBase {

  /**
   * {@inheritdoc}
   */
  public function getSettingsForm($values = []) {

    $form = [];
    $form['host'] = [
      '#type' => 'textfield',
      '#default_value' => $values['host'] ?? '',
      '#title' => $this->t('Host'),
      '#required' => TRUE,
    ];

    $form['user'] = [
      '#type' => 'textfield',
      '#default_value' => $values['user'] ?? '',
      '#title' => $this->t('User'),
      '#required' => TRUE,
    ];

    $form['password'] = [
      '#type' => 'password',
      '#default_value' => $values['password'] ?? '',
      '#title' => $this->t('Password'),
      '#required' => TRUE,
    ];

    $form['remote_dir'] = [
      '#type' => 'textfield',
      '#default_value' => $values['remote_dir'] ?? '',
      '#title' => $this->t('Remote Dir'),
    ];

    return $form;

  }

  /**
   * {@inheritdoc}
   */
  public function establishConnection($settings) {
    $this->connection = ftp_connect($settings['host']);

    if (ftp_login($this->connection, $settings['user'], $settings['password'])) {
      if ($settings['remote_dir']) {
        ftp_chdir($this->connection, $settings['remote_dir']);
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function collectFiles() {
    $remoteDirectory = ftp_pwd($this->connection);
    ftp_pasv($this->connection, TRUE);
    $ftpRawList = ftp_rawlist($this->connection, $remoteDirectory, TRUE);
    $files = [];
    $dir = '';
    foreach ($ftpRawList as $item) {

      if (substr($item, 0, strlen($remoteDirectory)) == $remoteDirectory) {
        $dir = trim(substr($item, strlen($remoteDirectory)), ':');
      }

      if ($item && $item[0] == '-') {
        $itemData = preg_split("/\s+/", $item);
        $file = end($itemData);
        if (count(explode('.', $file)) > 1 && explode('.', $file)[0]) {
          $files[] = [
            'path' => $dir . '/' . $file,
            'metadata' => [
              'modified_date' => ftp_mdtm($this->connection, $remoteDirectory . $dir . '/' . $file),
            ],
          ];
        }
      }
    }

    return $files;

  }

  /**
   * {@inheritdoc}
   */
  public function importFile($localFileName, $remoteFileName) {
    $remoteDirectory = ftp_pwd($this->connection);
    ftp_get($this->connection, $localFileName, $remoteDirectory . $remoteFileName);
  }

  /**
   * {@inheritdoc}
   */
  public function closeConnection() {
    ftp_close($this->connection);
  }

}
